@@ -35,21 +35,9 @@ enum subid_status {
 	SUBID_STATUS_ERROR = 3,
 };
 
-/*
- * subid_init: initialize libsubid
- *
- * @progname: Name to display as program.  If NULL, then "(libsubid)" will be
- *            shown in error messages.
- * @logfd:    Open file pointer to pass error messages to.  If NULL, then
- *            /dev/null will be opened and messages will be sent there.  The
- *            default if libsubid_init() is not called is stderr (2).
- *
- * This function does not need to be called.  If not called, then the defaults
- * will be used.
- *
- * Returns false if an error occurred.
- */
-bool subid_init(const char *progname, FILE *logfd);
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 /*
  * subid_get_uid_ranges: return a list of UID ranges for a user
@@ -151,5 +139,9 @@ bool subid_ungrant_uid_range(struct subordinate_range *range);
  */
 bool subid_ungrant_gid_range(struct subordinate_range *range);
 
+#ifdef __cplusplus
+}
+#endif
+
 #define SUBID_NFIELDS 3
 #endif
